class FetchConferenceDataJob < ApplicationJob
  queue_as :default

  def perform(conference_slug)
    conference = Conference.find_by(slug: conference_slug)

    if conference.import_in_progress? && conference.last_import_started_at > 5.minutes.ago
      Rails.logger.info "Skipping import for #{conference_slug} as another import is already in progress"
      return
    end

    import_history = ImportHistory.create!(
      conference: conference,
      started_at: Time.current,
      status: :started
    )

    conference.update(last_import_started_at: Time.current, last_import_error: nil)

    begin
      stages_before = conference.stages.count
      sessions_before = conference.sessions.count
      speakers_before = conference.speakers.count

      job_class_path = "#{conference.import_job_class.camelize}"
      if Object.const_defined?(job_class_path)
        job_class = job_class_path.constantize
        job_class.perform_now(conference_slug)
      else
        raise "No custom jobs for conference #{conference_slug} found"
      end

      stages_after = conference.stages.count
      sessions_after = conference.sessions.count
      speakers_after = conference.speakers.count

      import_history.update(
        completed_at: Time.current,
        status: :completed,
        stages_count: stages_after - stages_before,
        sessions_count: sessions_after - sessions_before,
        speakers_count: speakers_after - speakers_before
      )

      conference.update(last_import_completed_at: Time.current)
    rescue => e
      error_message = "#{e.class.name}: #{e.message}"
      backtrace = e.backtrace.join("\n")
      full_error = "#{error_message}\n\n#{backtrace}"

      import_history.update(
        status: :failed,
        error: full_error
      )

      conference.update(last_import_error: full_error)

      Rails.logger.error("Import failed for #{conference_slug}: #{error_message}")
      Rails.logger.error(backtrace)

      if conference.should_retry_import?
        backoff_time = [ 30, 60, 120 ][conference.retry_count].seconds
        FetchConferenceDataJob.set(wait: backoff_time).perform_later(conference_slug)
        Rails.logger.info("Scheduled retry for #{conference_slug} in #{backoff_time} seconds")
      end

      raise e
    end
  end
end
